/*******************************************************************************
* Author:					Thomas Fischer
* Email:					thomas-fischer25 t gmx.de
* Erstellungsdatum:			31.07.2009
* Letzte nderung:			14.06.2011
* Dateiname:				uartH.
* Beschreibung:    			Kommunikation ber UART mit Puffer und #-Trennung
* 
* Datum:     Autor:      	Grund der Aenderung:
* 06.08.2010 Fischer    	bernahme des neuen Headers mit Grund der Aenderung
*							Einfhrung der uart_debug Variable
*							F_CPU definieren
* 15.04.2011 Fischer		Einfhren des #-getrennten Sendeverfahrens.
*							Entfernen der nicht mehr bentigten uart_debug
*							Variable und der pck-Funktionen
* 05.05.2011 Fischer		Funktion uart_binary
* 14.06.2011 Fischer		Receive and Transmit Buffer with different size.
********************************************************************************/


#include <avr/io.h>
#include <avr/interrupt.h>
#include "uartH.h"

uint8_t transmitBuffer[UART_TRA_BUFFER_SIZE];
uint8_t *p_transmitRead = transmitBuffer;
uint8_t *p_transmitWrite = transmitBuffer;

uint8_t receiveBuffer[UART_REC_BUFFER_SIZE];
uint8_t *p_receiveRead = receiveBuffer;
uint8_t *p_receiveWrite = receiveBuffer;

#define UART_CLEAR_STATUS uart_status = 0
#define UART_SET_RECEIVEBUFFER_OVERFLOW (uart_status |= 1)
#define UART_SET_TRANSMITBUFFER_OVERFLOW (uart_status |= 2)
#define UART_STATE_RECEIVEBUFFER_OVERFLOW (uart_status & 1)
#define UART_STATE_TRANSMITBUFFER_OVERFLOW (uart_status & 2)

uint8_t uart_status = 0;
uint8_t maxTransmit = 0;
uint8_t maxReceive = 0;


/***********************************************************************************************************
*********** Empfangen **************************************************************************************
***********************************************************************************************************/

//Daten werden empfangen und in Eingangspuffer geschrieben. berlauf wird erkannt. Daten werden verworfen.
ISR(USART_RXC_vect) // Datenempfangsinterupt
{
	uint8_t receiveStorage = UDR;
	uint8_t *p_temp = p_receiveWrite + 1;
	
	if (p_temp >= receiveBuffer + UART_REC_BUFFER_SIZE)
		p_temp = receiveBuffer;
		
	if (p_temp == p_receiveRead)
		UART_SET_RECEIVEBUFFER_OVERFLOW;
	else
	{		
		p_receiveWrite = p_temp;
		*p_receiveWrite = receiveStorage;
	}
	return;
}


// Zeichen aus Eingangspuffer lesen. berlauf im Puffer wird erkannt, Daten werden verworfen.
uint8_t uart_getc(uint8_t *databyte)
{
	if (p_receiveRead == p_receiveWrite) return 0;	// Puffer leer
	
	int8_t result = p_receiveWrite - p_receiveRead;
	if (result < 0)
		result += UART_REC_BUFFER_SIZE;
	
	if (result > maxReceive)
		maxReceive = result;
	
	if (++p_receiveRead >= receiveBuffer + UART_REC_BUFFER_SIZE)
		p_receiveRead = receiveBuffer;
		
	*databyte = *p_receiveRead;
	return 1;
}

// Momentanes Zeichen lesen, Leseposition halten.
uint8_t uart_getcc(uint8_t *databyte)
{
	if (p_receiveRead == p_receiveWrite) return 0;
	uint8_t *p_temp = p_receiveRead + 1;
	if (p_temp >= receiveBuffer + UART_REC_BUFFER_SIZE)
		p_temp = receiveBuffer;
		
	*databyte = *p_temp;
	return 1;
}

// Lesezeiger weitersetzen
void uart_nextc(void)
{
	if (p_receiveRead == p_receiveWrite) return;
	if (++p_receiveRead >= receiveBuffer + UART_REC_BUFFER_SIZE)
		p_receiveRead = receiveBuffer;
	return;
}

/***********************************************************************************************************
*********** Senden *****************************************************************************************
***********************************************************************************************************/

//Einzelnes Byte in Ausgangspuffer schreiben. berlauf im Puffer wird erkannt, Daten werden verworfen.
void uart_writec(int8_t databyte)
{
	uint8_t *p_temp = p_transmitWrite + 1;
	
	if (p_temp >= transmitBuffer + UART_TRA_BUFFER_SIZE)
		p_temp = transmitBuffer;
		
	if (p_temp == p_transmitRead)
		UART_SET_TRANSMITBUFFER_OVERFLOW;
	else
	{	
		UCSRB &= ~(1<<UDRIE); 				//UDRE Interupt deaktivieren (UART Data Register Empty)
		p_transmitWrite = p_temp;
		*p_transmitWrite = databyte;
		UCSRB |= (1<<UDRIE);				//UDRE Interupt aktivieren (UART Data Register Empty)
	}
	return;
								
}

// String in den Ausgangspuffer schreiben.
void uart_writes(char *text)
{
	while(*text)
	{
		uart_writec(*text);
		text++;
	}
}


// schreibt eine Zahl mit beliebig vielen Stellen
void uart_zahlX(uint8_t digits, uint32_t temp1)
{
	char ziffern[11] = {0};
	unsigned char i;
	
	for(i=digits; i>0; i--)
	{
		ziffern[i-1] = '0' + (temp1 % 10);
		temp1 /= 10;	
	}

	uart_writes(ziffern);
}

// schreibt Bitfolge (8 Bit)
void uart_binary(uint8_t byte)
{
	char ziffern[11] = {'0','b',0,0,0,0,0,0,0,0,0};
	
	unsigned char i;
	
	for(i=10; i>2; i--)
	{
		ziffern[i-1] = '0' + (byte & 0x01);
		byte = (byte>>1);
		
	}

	uart_writes(ziffern);
}


//Daten werden aus Ausgangspuffer ausgelesen und versendet.
ISR(USART_UDRE_vect) 					// Datenregister ist leer und kann nun neu beschrieben werden 	
{										// (UART Data Register Empty) 
	
	if (p_transmitRead == p_transmitWrite)	// Puffer leer 
	{
		UCSRB &= ~(1<<UDRIE); 				// UDRE Interupt deaktivieren (UART Data Register Empty)
		return;
	}
	
	// Pufferausnutzung beobachten
	int8_t result = p_transmitWrite - p_transmitRead;
	if (result < 0)
		result += UART_TRA_BUFFER_SIZE;
	if (result > maxTransmit)
		maxTransmit = result;
		
		
	
	if (++p_transmitRead >= transmitBuffer + UART_TRA_BUFFER_SIZE)
		p_transmitRead = transmitBuffer;
		
	UDR = *p_transmitRead;
}


void uart_init(uint32_t baudrate)
{
	// USART Konfiguration
	uint32_t baudrate8 = baudrate << 3;
	uint32_t baudrate16 = baudrate8 << 1;
	uint16_t ubrr = ((F_CPU + baudrate8) / baudrate16) - 1;

	UBRRH = (ubrr >> 8) & 0xFF;
	UBRRL = ubrr & 0xFF;

	UCSRC |=((1<<URSEL)|(7<<UCSZ0)); // @Atmega8 setzt URSEL, USBS (durch 7<<UCSZ0) ->2Stopbits sowie UCSZ0 und UCSZ1 fr 8Bit bertragung
	//UCSRC |=((7<<UCSZ0)); //@Attiny2313  setzt USBS (durch 7<<UCSZ0) ->2Stopbits sowie UCSZ0 und UCSZ1 fr 8Bit bertragung
	UCSRB |=((1<<RXCIE)|(1<<TXEN)|(1<<RXEN)); 
	
	#if defined (__AVR_ATmega32__)
		DDRD  |= (1<<PD1);
		PORTD |= (1<<PD0);
	#endif
	
	
	return;
}